from books.models import *
from django.contrib import admin
from django.contrib.auth.models import User

def showCategories(obj):
  cats = obj.categories.iterator()
  s = ''
  for c in cats:
    s += c.name + " "
  return s

def labelID(obj):
  s = ''
  if obj.box:
    try:
      loc = Location.objects.get(pk=obj.box.id)
      s += loc.code + "/"
    except:
      s += ""
    s += obj.box.box_no + "/"
#  if obj.box.location:
#    s += obj.box.location.code + "/"
  s += str(obj.id)      
  return s

class BookImageInline(admin.TabularInline):
  model = Image
  extra = 3

class BookAdmin(admin.ModelAdmin):
  ordering = ('title',)
  search_fields = ['title', 'description']
  inlines = [BookImageInline]
  save_on_top = True;

class HardwareAdmin(admin.ModelAdmin):
  ordering = ('-id',)
  list_display = ('title', labelID, 'box', 'category', 'tags',)
  list_filter = ('category', 'box', 'tags',)
  list_editable = ('box', 'tags',)
  search_fields = ['title', 'description',]
  save_as = True
  save_on_top = True
  radio_fields = {'category': admin.HORIZONTAL}

class HardwareInline(admin.StackedInline):
  model = Hardware
  extra = 5
  save_on_top = True;

class BoxAdmin(admin.ModelAdmin):
  ordering = ('-id',)
  search_fields = ['box_no', 'description']
  list_display = ('box_no', 'location', 'tags',)
  inlines = [HardwareInline]
  list_filter = ('location',)
  list_editable = ('location', 'tags',)
  save_on_top = True;

class CatalogueAdmin(admin.ModelAdmin):
  ordering = ('title',)
  search_fields = ['title', 'artist']
  save_on_top = True;

class Link(admin.ModelAdmin):
  pass

class HardwareCategoryAdmin(admin.ModelAdmin):
  pass

admin.site.register(Hardware, HardwareAdmin) 
admin.site.register(Box, BoxAdmin) 
admin.site.register(HardwareCategory, HardwareCategoryAdmin)
#admin.site.register(Link)
admin.site.register(HardwareLink) 
admin.site.register(Catalogue, CatalogueAdmin) 
admin.site.register(Book, BookAdmin) 
admin.site.register(Owner)
admin.site.register(Location)

