#!/usr/bin/python

import sys
import types
import time
import string
import fileinput
import re
import urllib
import isbnvalidate
from dateutil.parser import parse as parseDate
import elementtree.ElementTree as ET
import feedparser
import simplejson


def myDate(datestr):
  dateObj = parseDate(datestr)
  month = dateObj.month
  yr = dateObj.year
  d = {
        'year': yr,
        'month': month
      }
  return d

def getHtmlFromUrl(url):
  try:
    u = urllib.urlopen(url)
    t = u.read()
    u.close
  except:
    return False
  return t

def getimagesrc(isbn):
    if (len(isbn) == 13):
        newisbn = isbnvalidate.convertISBN13toISBN10(isbn)
        isbn = newisbn.strip()

    inf = urllib.urlopen('http://www.amazon.com/gp/product/images/' + isbn)
    allhtml = inf.read()
    inf.close()
    imghtml = re.search('<div id="imageViewerDiv"><img src="(.*?)" id="prodImage" /></div>', allhtml)

    imgsrc1 = imghtml.group(1)
    return imgsrc1



#TODO: Try avoiding wrapping entire search in 'try'.
def ChaptersCa(isbn):
  url = 'http://www.chapters.ca/books/details/default.asp?ISBN=' + isbn
  html = getHtmlFromUrl(url)
  if not html:
    return False
  if (string.find(html, 'chapters.indigo.ca: Error:') >= 0):
    return False
  titlehtml = re.search('<h1>(.*?)</h1>', html)
  authorhtml = re.search('<p class="link">Author: (<a href=.*</a>)</p>', html)
  dimensionshtml = re.search('<label>Dimensions:</label>(.*?)</p>', html)
  publisherhtml = re.search('<label>Publisher:</label>(.*?)</p>', html)
  publishedhtml = re.search('<label>Published:</label>(.*?)</p>', html)
  #print (titlehtml, authorhtml, dimensionshtml, publisherhtml, publishedhtml)
  try:
    authorlinks = authorhtml.group(1).strip()
    authors = re.findall('<a .*?>(.*?)</a>', authorlinks) # returns list of authors

    for a in authors:
      if re.search('See more ', a):
        tmp = a
    authors.remove(a)
    d = {}
  
    d['source'] = "ChaptersCa"
    d['pages'] = dimensionshtml.group(1).split(",")[0].strip()
    d['dimensions'] = dimensionshtml.group(1).split(",")[1].strip()
  #      dimensions = dimensionshtml.group(1).strip() 
    d['publisher'] = publisherhtml.group(1).strip()
    d['published'] = myDate(publishedhtml.group(1).strip())
    d['title'] = titlehtml.group(1).strip()
    d['coverimg'] = getimagesrc(isbn)  
    d['url'] = url
    d['authors'] = authors
  except:
    return False
  return d

#TODO: Regex failing. Test.
def Amazon(isbn):
  url = 'http://www.amazon.com/gp/search/ref=sr_adv_b/?field-isbn=' + isbn
  html = getHtmlFromUrl(url)
  if not html:
    return False
  if (string.find(html, 'Your search did not match any products.') >= 0):
    return False
  titlehtml = re.search('<span id="btAsinTitle" style="">(.*?)</span>', html)
  paperbackhtml = re.search('<li><b>Paperback:</b>(.*?)</li>', html)
  weighthtml = re.search('<li><b>Shipping Weight:</b>(.*?)\(.*?\)</li>', html)
  publisherhtml = re.search('<li><b>Publisher:</b>(.*?) \((.*?)\)</li>', html)
  authors = re.findall('<a href=".*?">(.*?)</a> \(Author\)', html) # returns list of authors
  try: 
    d = {}
    d['source'] = 'Amazon'
    d['pages'] = paperbackhtml.group(1).strip()
    d['dimensions'] = weighthtml.group(1).strip()
    d['publisher'] = publisherhtml.group(1).strip()
    d['published'] = myDate(publisherhtml.group(2).strip())
    d['title'] = titlehtml.group(1).strip()
    d['coverimg'] = getimagesrc(isbn)
    d['url'] = url
  except:
    return False
#  sourceurl = 'http://www.amazon.com/gp/search/ref=sr_adv_b/?field-isbn=' + isbn   
  return d

#TODO: Dont return False if a single key does not exist
def ISBNdb(isbn):
  ISBNdb_KEY = 'HLRQD9WN'
# http://isbndb.com/api/books.xml?access_key=HLRQD9WN&index1=isbn&value1=1400082463
  url = 'http://isbndb.com/api/books.xml?access_key=' + ISBNdb_KEY + '&index1=isbn&value1=' + isbn
  try:
    xml = getHtmlFromUrl(url)
    tree = ET.XML(xml)    
  except:
    return False
  results = tree.find("BookList").get("total_results")
  if (results < 1):
    return False
  d = {}
  d['source'] = 'ISBNdb'
  try:
    book = tree.getchildren()[0].getchildren()[0]
    d['title'] = book.findtext("Title")
    titleLong = book.findtext("TitleLong").strip()
    if (titleLong != ''):
      d['title'] = titleLong
    d['publisher'] = book.findtext("PublisherText")
    d['summary'] = book.findtext("Summary")
    d['notes'] = book.findtext("Notes")
    authorsTxt = book.findtext("AuthorsText")
    d['authors'] = authorsTxt.split(",")
  except:
    return False 
  return d

#TODO: Fetch weblinks.
#TODO: Clean-up to not go to exception when can't find a single attribute
#TODO: Go through XML and fetch more relevant fields.
#TODO: Make URL more specific to look for ISBN codes. [IMP!!] 
def GoogleBooks(isbn):
  url = 'http://books.google.com/books/feeds/volumes?q=' + isbn
  try:
    feed = feedparser.parse(url)
    book = feed.entries[0]
  except:
    return False
  try:
    d = {}
    d['source'] = 'GoogleBooks'
    d['title'] = book.title
  #  d['summary'] = book.description
    d['pages'] = book.dc_format
    d['author'] = [book.author]
    d['publisher'] = book.publisher
  except:
    return False
  return d

#TODO: Don't return book! Return dict according to our fields.
#TODO: Fetch Chapters in a meaningful way
#TODO: Add relevant fields - first_sentence, etc.
def OpenLibrary(isbn):
  if (len(isbn) == 10):
    url1 = 'http://openlibrary.org/api/things?query={%22type%22%3A+%22\%2Ftype\%2Fedition%22%2C+%22isbn_10%22%3A+%22' + isbn + '%22}&prettyprint=true&text=true'
  elif (len(isbn) == 13):
    url1 = 'http://openlibrary.org/api/things?query={%22type%22%3A+%22\%2Ftype\%2Fedition%22%2C+%22isbn_13%22%3A+%22' + isbn + '%22}&prettyprint=true&text=true'
  else:
    return False
  json = getHtmlFromUrl(url1)
  print json
  if not json:
    return False  
  j = simplejson.loads(json) 
  if (j['status'] != 'ok' or len(j['result']) == 0):
    return False
  olId = j['result'][0]
  url2 = "http://openlibrary.org/api/get?key=" + olId
  json2 = getHtmlFromUrl(url2)
  book = simplejson.loads(json2)
  return book

#TODO: write regex parser for LookUpByIsbn to return esp. Reviews
def LookUpByIsbn(isbn):
  url = "http://www.lookupbyisbn.com/itemDetail.aspx?type=Books&id=" + isbn
  return False


def searchAll(isbn, **kwargs):
  sources = [OpenLibrary, GoogleBooks, ISBNdb, Amazon, ChaptersCa]
  return getAggregate(sources, isbn)

#TODO: Handle specific cases of 'author' and 'weblink' fields. Also, other lists, like Chapters.
def getAggregate(sources, isbn, **kwargs):
  if not isbnvalidate.isValidISBN(isbn):
    return {'errors': "Invalid ISBN"}
#  if kwargs['isbnformat'] == 10 and len(isbn) == 13:
#    isbn = isbnvalidate.convertISBN13toISBN10(isbn)
#  if kwargs['isbnformat'] == 13 and len(isbn) == 10:
#    isbn = isbnvalidate.convertISBN10toI3(isbn)

  results = []
  for source in sources:
    rDict = source(isbn)
    if (rDict):
      results.append(rDict)
  dFinal = {}
  for result in results:
    for key in r.keys():
      if (key not in dFinal.keys()):
        dFinal[key] = result[key]
  return dFinal
      

     

  
