#!/usr/bin/python
# Usage: searchAll(isbn) with isbn passed as a string.
# Returns values defined in returnDict

import sys
import types
import time
import string
import fileinput
import re
import urllib
import isbnvalidate
from dateutil.parser import parse as parseDate
import elementtree.ElementTree as ET
import feedparser
import simplejson

# ISBNlist = [ '9780596001322', '9781590593912', '9780596003135', '0596102062', '0131103628', '1905789041', '1400082463' ]

def searchAll(isbn, **kwargs):
  sources = [OpenLibrary, GoogleBooks, ISBNdb, Amazon, ChaptersCa]
#  sources = [Amazon]
  return getAggregate(sources, isbn)

def getAggregate(sources, isbn, **kwargs):
  if not isbnvalidate.isValidISBN(isbn):
    return {'errors': "Invalid ISBN"}
  returnDict = {
    'title': u'',
    'titleLong': u'',
    'authors': [],
    'weblinks': [],
    'published': {},
    'publisher': u'',
    'isbn': u'',
    'first_sentence': u'',
    'chapters': [],
    'netReviews': [],
    'summaries': [],
    'pages': 0,
    'dimensions': u'',
    'weight': 0,
    'relatedBooks': [],
    'tags': [],
    'sources': [],
    'imageUrls': [],
    'coverImage': u''  
    }
#  if kwargs['isbnformat'] == 10 and len(isbn) == 13:
#    isbn = isbnvalidate.convertISBN13toISBN10(isbn)
#  if kwargs['isbnformat'] == 13 and len(isbn) == 10:
#    isbn = isbnvalidate.convertISBN10toI3(isbn)
  results = []
  for source in sources:
    rDict = source(isbn)
    if (rDict):
      results.append(rDict)
  lists = []
#  lists = ['authors', 'weblinks', 'chapters', 'netReviews', 'summaries', 'relatedBooks', 'source']
  for result in results:
    for key in result.keys():
      if returnDict.has_key(key):
        if isinstance(result[key], list):
          if key not in lists:
            lists.append(key)
          for r in result[key]:
            returnDict[key].append(r)
        else:
          returnDict[key] = result[key]
  for l in lists:
    if len(returnDict[l]) > 1:
      returnDict[l] = unique(returnDict[l])
  returnDict['isbn'] = isbn
  returnDict['coverImage'] = getCoverImageUrl(isbn)     

  return returnDict

      
def ChaptersCa(isbn):
  url = 'http://www.chapters.ca/books/details/default.asp?ISBN=' + isbn
  html = getHtmlFromUrl(url)
  if not html:
    return False
  if (string.find(html, 'chapters.indigo.ca: Error:') >= 0):
    return False
  titlehtml = re.search('<h1>(.*?)</h1>', html)
  authorhtml = re.search('<p class="link">Author: (<a href=.*</a>)</p>', html)
  dimensionshtml = re.search('<label>Dimensions:</label>(.*?)</p>', html)
  publisherhtml = re.search('<label>Publisher:</label>(.*?)</p>', html)
  publishedhtml = re.search('<label>Published:</label>(.*?)</p>', html)

  #print (titlehtml, authorhtml, dimensionshtml, publisherhtml, publishedhtml)
  try:
    authorlinks = authorhtml.group(1).strip()
    authors = re.findall('<a .*?>(.*?)</a>', authorlinks) # returns list of authors
    i = 0
    for a in authors:
      if re.search('See more ', a):
        authors.remove(a)
      else:
        authors[i] = a.strip()
        if (authors[i] == ''):
          authors.remove(a)
      i += 1
  except:
    authors = []

  d = {}
  d['sources'] = [{'name': 'ChaptersCa', 'url': url}]
  
  try:    
    d['pages'] = dimensionshtml.group(1).split(",")[0].strip()
    d['dimensions'] = dimensionshtml.group(1).split(",")[1].strip()
  except:
    pass
#      dimensions = dimensionshtml.group(1).strip() 
  try:
    d['publisher'] = publisherhtml.group(1).strip()
  except:
    pass

  try:    
    d['published'] = myDate(publishedhtml.group(1).strip())
  except:
    pass
  
  try:
    d['title'] = titlehtml.group(1).strip()
  except:
    pass
  
  try:
    d['imageUrls'] = [getimagesrc(isbn)]  
  except:
    pass    

  d['authors'] = authors
  return d

def Amazon(isbn):
  url = 'http://www.amazon.com/gp/search/ref=sr_adv_b/?field-isbn=' + isbn
  html = getHtmlFromUrl(url)
  if not html:
    return False
  if (string.find(html, 'Your search did not match any products.') >= 0):
    return False
  titlehtml = re.search('<span id="btAsinTitle" style="">(.*?)</span>', html)
  paperbackhtml = re.search('<li><b>Paperback:</b>(.*?)</li>', html)
  weighthtml = re.search('<li><b>Shipping Weight:</b>(.*?)\(.*?\)</li>', html)
  publisherhtml = re.search('<li><b>Publisher:</b>(.*?) \((.*?)\)</li>', html)
  authors = re.findall('<a href=".*?">(.*?)</a> \(Author\)', html) # returns list of authors
  d = {}
  d['sources'] = [{'name': 'Amazon', 'url': url}]
  try:          
    d['pages'] = paperbackhtml.group(1).strip()
  except:
    pass
  try:
    d['dimensions'] = weighthtml.group(1).strip()
  except:
    pass
  try:
    d['publisher'] = publisherhtml.group(1).strip()
  except:
    pass
  try:
    d['published'] = myDate(publisherhtml.group(2).strip())
  except:
    pass
  try:
    d['title'] = titlehtml.group(1).strip()
  except:
    pass
#  sourceurl = 'http://www.amazon.com/gp/search/ref=sr_adv_b/?field-isbn=' + isbn   

  d['authors'] = authors

  return d


def ISBNdb(isbn):
  ISBNdb_KEY = 'HLRQD9WN'
# http://isbndb.com/api/books.xml?access_key=HLRQD9WN&index1=isbn&value1=1400082463
  url = 'http://isbndb.com/api/books.xml?access_key=' + ISBNdb_KEY + '&index1=isbn&value1=' + isbn
  try:
    xml = getHtmlFromUrl(url)
    tree = ET.XML(xml)    
  except:
    return False
  results = tree.find("BookList").get("total_results")
  if (results < 1):
    return False
  d = {}
  d['sources'] = [{'name': 'ISBNdb', 'url': url}]
  try:
    book = tree.getchildren()[0].getchildren()[0]
  except:
    return False

  try:
    d['titleLong'] = book.findtext("TitleLong").strip()
  except:
    pass

  try:
    d['title'] = book.findtext("Title")
  except:
    pass

  try:
    d['publisher'] = book.findtext("PublisherText")
  except:
    pass

  try:
    d['summaries'] = [book.findtext("Summary")]
  except:
    pass

  try:
    d['notes'] = book.findtext("Notes")
  except:
    pass

  try:
    authors = book.findtext("AuthorsText").split(",")
    i = 0
    for a in authors:
      if a.strip() == '':
        authors.remove(a)
  except:
    pass
 
  return d

#TODO: Make URL more specific to look for ISBN codes. [IMP!!] - Alden 
def GoogleBooks(isbn):
  url = 'http://books.google.com/books/feeds/volumes?q=' + isbn
  try:
    feed = feedparser.parse(url)
    book = feed.entries[0]
  except:
    return False
  d = {}
  d['sources'] = [{'name': 'GoogleBooks', 'url': url}]
  try:    
    d['title'] = book.title
  except:
    pass
  try:
    d['summaries'] = [book.dc_description]
  except:
    pass
  try:
    d['pages'] = book.dc_format
  except:
    pass
  try:
    d['author'] = [book.dc_creator]
  except:
    pass
  try:
    d['publisher'] = book.publisher
  except:
    pass

  try:
    d['published'] = myDate(book.dc_date)
  except:
    pass

  try:
    d['weblinks'] = []
    for link in book['links']:
      l = {'url': link['href'], 'description': link['rel'] }
      d['weblinks'].append(l)
  except:
    pass

  return d

def OpenLibrary(isbn):
  # 'http://openlibrary.org/api/things?query={%22type%22%3A+%22\%2Ftype\%2Fedition%22%2C+%22isbn_13%22%3A+%221400082463%22}&prettyprint=true&text=true'
  if (len(isbn) == 10):
    url1 = 'http://openlibrary.org/api/things?query={%22type%22%3A+%22\%2Ftype\%2Fedition%22%2C+%22isbn_10%22%3A+%22' + isbn + '%22}&prettyprint=true&text=true'
  elif (len(isbn) == 13):
    url1 = 'http://openlibrary.org/api/things?query={%22type%22%3A+%22\%2Ftype\%2Fedition%22%2C+%22isbn_13%22%3A+%22' + isbn + '%22}&prettyprint=true&text=true'
  else:
    return False
  json = getHtmlFromUrl(url1)
#  print json
  if not json:
    return False  
  j = simplejson.loads(json) 
  if (j['status'] != 'ok' or len(j['result']) == 0):
    return False
  olId = j['result'][0]
  url2 = "http://openlibrary.org/api/get?key=" + olId

  try:
    json2 = getHtmlFromUrl(url2)
    book = simplejson.loads(json2)
    r = book['result']
  except:
    return False

  d = {}
  d['sources'] = [{'name': "OpenLibrary", 'url': url2}]

  try:
    d['first_sentence'] = r['first_sentence']['value']
  except:
    pass

  try:
    d['pages'] = r['number_of_pages']
  except:
    pass

  try:
    d['dimensions'] = r['physical_dimensions']
  except:
    pass

  try:
    d['published'] = myDate(r['publish_date'])
  except:
    pass

  chapters = []
  try:    
    for chapter in r['table_of_contents']:
      c = {}
      c['title'] = chapter['title']
      c['synopsis'] = ''
      c['link'] = ''
      chapters.append(c)
    d['chapters'] = chapters
  except:
    pass

  try:
    d['weight'] = r['weight']
  except:
    pass

  tags = []
  try:
    for tag in r['subjects']:
      tags.append(tag)
    d['tags'] = tags
  except:
    pass
  
  try:
    d['titleLong'] = r['subtitle']
  except:
    pass
  
  return d

#TODO: write regex parser for LookUpByIsbn to return esp. Reviews. - Alden (low priority)
def LookUpByIsbn(isbn):
  url = "http://www.lookupbyisbn.com/itemDetail.aspx?type=Books&id=" + isbn
  return False

#TODO: Write regex parser for CB-India computer books. -Alden (low priority)
def CBIndia(isbn):
  return False

#TODO: This seems like a fairly large collection of Indian books
def FirstAndSecond(isbn):
  url = "http://www.firstandsecond.com/store/books/info/search.asp?styp=isb&isb=" + isbn
  return False


def myDate(datestr):
  dateObj = parseDate(datestr)
  month = dateObj.month
  yr = dateObj.year
  day = dateObj.day
  d = {
        'year': yr,
        'month': month,
        'day': day
      }
  return d


def getHtmlFromUrl(url):
  try:
    u = urllib.urlopen(url)
    t = u.read()
    u.close
  except:
    return False
  return t

def getCoverImageUrl(isbn):
    if (len(isbn) == 13):
        newisbn = isbnvalidate.convertISBN13toISBN10(isbn)
        isbn = newisbn.strip()
    try:
      inf = urllib.urlopen('http://www.amazon.com/gp/product/images/' + isbn)
      allhtml = inf.read()
      inf.close()
      imghtml = re.search('<div id="imageViewerDiv"><img src="(.*?)" id="prodImage" /></div>', allhtml)
      imgsrc1 = imghtml.group(1)
    except:
#      print "Unable to retrieve book cover image"
      return ''
    return imgsrc1

def unique(seq):
  checked = []
  for e in seq:
    if e not in checked:
      checked.append(e)
  return checked
