from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
# from django.contrib.auth import authenticate, login, logout
from books.models import *
from django.utils import simplejson
from django import forms
from django.forms.models import inlineformset_factory
from isbnlookup2 import searchAll
#from django.contrib.auth.messages import *
import datetime
import sys
from django.template import RequestContext
from dateutil.parser import parse as parseDate

class BookForm(forms.ModelForm):
  title = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}))
  authors = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}), required=False)
  dimensions = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}), required=False)
  publisher = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}), required=False)
  published = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}), required=False)
  pages = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}), required=False)
  weight = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}), required=False)
  first_sentence = forms.CharField(widget = forms.TextInput(attrs = {'class': 'autoisbn'}), required=False)
  tags = forms.CharField(widget = forms.TextInput(attrs = {'size': '50'}), required=False)
  autotags = forms.CharField(widget = forms.TextInput(attrs = {'size': '50', 'class' : 'autoisbn'}), required=False)

  class Meta:
    model = Book

def dateformat(datestr):                                            
   return datetime.date(*time.strptime(datestr, "%B %d, %Y")[0:3]).isoformat()

def index(request):
  books = Book.objects.all()

  return render_to_response("index.html", {'books': books})

#  return render_to_response("index.html", context_instance=RequestContext(request))

@login_required
def add(request):
  linksInline = inlineformset_factory(Book, Weblink)
  reviewsInline = inlineformset_factory(Book, WebReview)
  summariesInline = inlineformset_factory(Book, Summary)
  relatedBookInline = inlineformset_factory(Book, RelatedBook)
  imageUrlInline = inlineformset_factory(Book, ImageUrl)
  chapterInline = inlineformset_factory(Book, ChapterEntry)
  sourcesInline = inlineformset_factory(Book, Source)
#  
  if (request.method == 'POST'):
    bookform = BookForm(request.POST)
#    bookform.published = dateformat(bookform.published)
    linksFormset = linksInline(request.POST, prefix='weblinks')
    reviewsFormset = reviewsInline(request.POST, prefix='webreviews')
    summariesFormset = summariesInline(request.POST, prefix='summaries')
    relatedBookFormset = relatedBookInline(request.POST, prefix='relatedbooks')
    imageUrlFormset = imageUrlInline(request.POST, prefix='imageurls')
    chapterFormset = chapterInline(request.POST, prefix='chapters')
    sourcesFormset = sourcesInline(request.POST, prefix='sources')
    if bookform.is_valid():
      newbook = bookform.save(commit=False)
      newbook.added_by = request.user
      request.session['owner'] = newbook.owner.id
      request.session['location'] = newbook.location.id
      newbook.published = parseDate(bookform.cleaned_data['published'])
      newbook.save()
      astring = bookform.cleaned_data['authors']
      alist = astring.split(';')
      authors = []
      for a in alist:
        if (a.strip() != ''):
          au = Author(name = a)
          au.save()
          authors.append(au)
      newbook.authors = authors
#      weblinks = linksFormset.save()
#      reviews = reviewsFormset.save()
#      summaries = summariesFormset.save()
#      relatedBooks = relatedBookFormset.save()
#      imageUrls = imageUrlFormset.save()
#      chapters = chapterFormset.save()
#      newbook.weblinks = weblinks
      newbook.save()

      for f in linksFormset.forms:
        if f.is_valid():
          f.cleaned_data['book'] = newbook
          if f.cleaned_data.has_key('url'):
            f.save()

      for f in summariesFormset.forms:
        if f.is_valid():
          f.cleaned_data['book'] = newbook
          if f.cleaned_data.has_key('txt'):
            f.save()

      for f in relatedBookFormset.forms:
        if f.is_valid():
          f.cleaned_data['book'] = newbook
          if f.cleaned_data.has_key('url'):
            f.save()

      for f in imageUrlFormset.forms:
        if f.is_valid():
          f.cleaned_data['book'] = newbook
          if f.cleaned_data.has_key('url'):
            f.save()

      for f in chapterFormset.forms:
        if f.is_valid():
          f.cleaned_data['book'] = newbook
          if f.cleaned_data.has_key('link'):
            f.save()

      for f in sourcesFormset.forms:
        if f.is_valid():
          f.cleaned_data['book'] = newbook
          if f.cleaned_data.has_key('name'):
            f.save()

#          if f.cleaned_data['url'] != '':
#            f.save()
#        weblink = form.save(commit=False)
#        weblink.book = newbook
#        if form.is_valid():       
#          weblink = Weblink()
#          weblink['url'] = form.cleaned_data['url']
#          weblink['description'] = form.cleaned_data['description']
#          weblink['book'] = newbook
#          weblink.save()
#          weblink.book = thisBook
#          weblink.save()
      request.user.message_set.create(message="Your book was added successfully.")
      return HttpResponseRedirect("/add")      
  else:    
    linksFormset = linksInline(prefix='weblinks')
    reviewsFormset = reviewsInline(prefix='webreviews')
    summariesFormset = summariesInline(prefix='summaries')
    relatedBookFormset = relatedBookInline(prefix='relatedbooks')
    imageUrlFormset = imageUrlInline(prefix='imageurls')
    chapterFormset = chapterInline(prefix='chapters')
    sourcesFormset = sourcesInline(prefix='sources')  
    bookform = BookForm()
#    formsets = [linksFormset, chapterFormset]
  formsets = [linksFormset, reviewsFormset, summariesFormset, relatedBookFormset, imageUrlFormset, chapterFormset, sourcesFormset]
  defaultsD = {}
  if request.session.has_key('location'):
    defaultsD['location'] = request.session['location']
  if request.session.has_key('owner'):
    defaultsD['owner'] = request.session['owner']
#  print repr(defaultsD)
  return render_to_response("addbook.html", {'form': bookform, 'formsets': formsets, 'defaults': defaultsD}, context_instance=RequestContext(request))
  
def isbnLookup(request):
  isbn = request.GET.get('isbn')
  isbnDict = searchAll(isbn)
  if (request.is_ajax()):
    return HttpResponse(simplejson.dumps(isbnDict), mimetype='application/javascript')
  else:

#Ideally we would not have an if and else statement doing the same thing ;-)
    return HttpResponse(simplejson.dumps(isbnDict), mimetype='application/javascript')

def addLocation(request):
  if request.is_ajax():
    locationName = request.GET['name']
    l = Location(name = locationName)
    l.save()
    return HttpResponse(l.id)
  else:
    return HttpResponse(str(o.id))

def addOwner(request):
  if request.is_ajax():
    ownerName = request.GET['name']
    o = Owner(name = ownerName)
    o.save()
    return HttpResponse(str(o.id))
  else:
    return HttpResponse("you are not ajax.")
  

