from django.db import models
from django.contrib.auth.models import User
from books.models import Owner, Location
from tagging.fields import TagField
from tagging.models import Tag
from books.fields import MultiSelectField, MultiSelectFormField

CATEGORY_CHOICES = (
    ('artworks', 'Artworks'),
    ('control', 'Control'),
    ('cables', 'Cables'),
    ('connectors', 'Connectors'),
    ('computer', 'Computer'),
    ('storage', 'Storage Devices'),
    ('av', 'Audio / Video Equipment'),
    ('electronics', 'Electronics'),
    ('electrical', 'Electrical & Lighting'),
    ('mechanical', 'Mechanical Parts'),
    ('media', 'Media'),
    ('tools', 'Tools'),
    ('supplies', 'Supplies'),
    ('other', 'Other'),
)

class Hardware(models.Model):
  title = models.CharField(max_length=255)
  model_number = models.CharField(max_length=255, blank=True)
  serial_number = models.CharField(max_length=255, blank=True)
  description = models.TextField(blank=True)
  owner = models.ForeignKey(Owner, blank=True)
  location = models.ForeignKey(Location, blank=True)
  tags = TagField("Tags", blank=True, help_text="Enter as many tags as you like, separated by commas")
  added_by = models.ForeignKey(User, editable=False)
  date_added = models.DateTimeField(auto_now=True, editable=False)
  category = MultiSelectField(max_length=355, choices=CATEGORY_CHOICES, verbose_name="Category")
        

  def __unicode__(self):
    return self.name

  class Meta:
    verbose_name_plural = "Hardware"

class Link(models.Model):
  name = models.CharField(max_length=255)
  url = models.URLField()

  def __unicode__(self):
    return self.name

