/*
*@class Formset class
*@property {String} prefix same prefix as defined in django view while instantiating formset.
*@property {String} prefixStr prefix string to 
*@property {String} jqPrefix just a helpful prefix with '#_id' prepended to use in jQuery selector strings
*@property {jQuery Object} totalFormsElem the Input field of the TOTAL_FORMS field for this formset. (Django: management_form)
*@property {jQuery Object} cloneObj a jQuery element that is a clone of a single form in the formset.
*@property {jQuery Object} titleElem The div that contains the title of the formset

*/

/*
*@constructor
*@param {String} Prefix for this formset (same as used in django during instantiation)
*/
var Formset = function(prefix) {
  var that = this;
  this.prefix = prefix;
  this.jqPrefix = '#id_' + prefix;
  this.prefixStr = 'id_' + this.prefix + '-';
  this.totalFormsElem = $(that.jqPrefix + '-TOTAL_FORMS');
  this.cloneObj = function() {
    var that = this;    
    return $('#' + that.prefix + ' > .formset_forms').children('.formset_form:first').clone()
    }
  this.titleElem = $('#' + that.prefix + ' > .formset_title');
  this.formsetElem = $('#' + that.prefix + ' > .formset_forms');
  this.titleElem.click(function() {
    that.formsetElem.toggle('slow', function() {
      thisTop = that.titleElem.offset().top;
      $(document).scrollTop(thisTop);
      });
    });
  }

/*
*Toggles display of formset (except for title element), also scrolls the page to the top of the title element.
* TODO: fix the scroll being weird or get rid of it.
*/
Formset.prototype.toggleDisplay = function() {
  var that = this;  
  that.formsetElem.toggle('slow', function() {
    thisTop = that.titleElem.offset().top;
    $(document).scrollTop(thisTop);
    });
  }

/*
* Get total no of forms in formset.
* @returns {Int} Total no of forms according to formset metadata.
*/
Formset.prototype.getTotal = function() {
  return parseInt(this.totalFormsElem.val());    
  }

/*
* Set total no of forms in formset.
*@param {Int} No of forms to set formset metadata to.
*/
Formset.prototype.setTotal = function(newNo) {
  this.totalFormsElem.val(newNo);
  return true;
  }

/*
* Increment formset metadata total by one
*/
Formset.prototype.increment = function() {
  var currTotal = this.getTotal();
  var newTotal = currTotal + 1;
  this.setTotal(newTotal);
  return true;
  }


Formset.prototype.getFieldName = function(idStr) {
//  var thisId = jqObj.attr('id');
//  var thisPrefixStr = 'id_' + this.prefix + '-';
  var that = this;
  var prefixLength = this.prefixStr.length + 2;
  var thisField = idStr.substring(prefixLength);
  console.log(that.prefixStr);
  return thisField;
  }

/*
*@returns {Array} returns array of strings representing field names present in this formset.
*/
Formset.prototype.getFields = function() {
  var that = this;
  var fields = [];
  that.cloneObj().find('input').each(function() {
    if ($(this).attr('type') != 'hidden') {
      var thisField = that.getFieldName($(this).attr('id'))
      if (thisField != 'DELETE') {      
        fields.push(thisField);     
        }  
      }
    });  
  that.cloneObj().find('textarea').each(function() {
    if ($(this).attr('type') != 'hidden') {
      var thisField = that.getFieldName($(this).attr('id'))
      if (thisField != 'DELETE') {      
        fields.push(thisField);
        }
      }
    });
  return fields;
  }

/*
* Add a form to the formset. Use this method from the outside.
* @param {Int} No of forms to add (Optional), defaults to 1.
*/
Formset.prototype.add = function(no) {
  if (!no) {
    var no = 1;
    }
  var that = this;
  for (var i=0; i < no; i++) {
    var e = this.cloneObj();
  //TODO: Figure out how to deal with changing 'for' attribute on labels correctly.
  /*
    e.find('label').each(function() {
      var newId = that.incrementId($(this).attr('for'));
  //    $(this).attr('id', newId);
      $(this).attr('for', newId); 
      });
  */
    e.find('input').each(function() {
      $(this).attr('id', that.incrementId($(this).attr('id')));
      $(this).attr('name', that.incrementName($(this).attr('id')));
      });
    e.find('textarea').each(function() {
      $(this).attr('id', that.incrementId($(this).attr('id')));
      $(this).attr('name', that.incrementName($(this).attr('id')));
      });
    this.increment();
    this.formsetElem.append(e);
    }
  return true;
  }

/*
*@param {String} id of element to increment
*@returns {String} incremented id
*/
Formset.prototype.incrementId = function(idStr) {
  var newTotal = this.getTotal() + 1;
  var newId = this.prefixStr + newTotal.toString() + '-' + this.getFieldName(idStr);
  return newId;          
  }

/*
*@param {String} name of element to increment
*@returns {String} incremented name
*/
Formset.prototype.incrementName = function(idStr) {
  var newTotal = this.getTotal() + 1;
  var newId = this.prefix + '-' + newTotal.toString() + '-' + this.getFieldName(idStr);
  return newId;
  }

/*
* Populates a formset with an array of JSON data
*@param {JSON Array} like so - [{'field1': 'some value', 'field2': 'some more value'}, {'field1': 'foo', 'field2': 'bar'}]
*@returns true once done
*/

Formset.prototype.populate = function(data) {
  var that = this;
  var noOfEntries = data.length;
  if (data.length > 0) {
//    that.titleElem.html(that.titleElem.html() + " " + noOfEntries.toString());
    that.titleElem.children('.formset_no_results').html(noOfEntries.toString());
    var totalFormsToShow = noOfEntries + 3;
    var newFormsToAdd = totalFormsToShow - this.getTotal();
    this.add(newFormsToAdd);
    var fields = this.getFields();
    for (var f in fields) {
      var fieldName = fields[f];
      for (var d in data) {
        if (data[d]) {
          var val = data[d][fieldName];
          $(that.jqPrefix + '-' + d.toString() + '-' + fieldName).val(val);
          }
        }
      } 
    }
  return true;
  }

